#!/usr/bin/perl -w
#
#
# check_disk.pl <host> <share> <user> <pass> [warn] [critical] [port]
#
# NetSaint host script to get the disk usage from a SMB share
#
# Changes and Modifications
# =========================
# 7-Aug-1999 - Michael Anthon
#			Created from check_disk.pl script provided with netsaint_statd
#			(basically cause I was too lazy (or is that smart?) to write it from scratch)
# 8-Aug-1999 - Michael Anthon
#			Modified [warn] and [critical] parameters to accept format of nnn[M|G] to
#			allow setting of limits in MBytes or GBytes.  Percentage settings for large
#			drives is a pain in the butt

require 5.004;
use POSIX;
use strict;

sub usage;

my $smbclient="/usr/bin/smbclient";
my $smbclientoptions="";
my $TIMEOUT = 15;

my %ERRORS = ('UNKNOWN', '-1',
		'OK', '0',
		'WARNING', '1',
		'CRITICAL', '2');
my $remote = shift || &usage(%ERRORS);
my $share = shift || &usage(%ERRORS);
my $user = shift || "guest";
my $passwd = shift || "guest";
my $warn = shift || 85;
my $crit = shift || 95;


my $state = "OK";
my $answer = undef;
my $res = undef;
my @lines = undef;
my $cmd = undef;


# Just in case of problems, let's not hang NetSaint
$SIG{'ALRM'} = sub { 
	print "No Answer from Client\n";
	exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);

#Execute an "ls" on the share using smbclient program, get the results into $res
$cmd = "$smbclient \/\/$remote\/$share $passwd -U $user $smbclientoptions -c ls";
$res = qx/$smbclient \/\/$remote\/$share $passwd -U $user $smbclientoptions -c ls/;
#Turn off alarm
alarm(0);

#Split $res into an array of lines
@lines = split /\n/, $res;
#Get the last line into $_
$_ = $lines[$#lines];
#print "$_\n";
#Process the last line to get free space.  
#If line does not match required regexp, return an UNKNOWN error
if (/\s*(\d*) blocks of size (\d*)\. (\d*) blocks available/) {

	my ($avail) = ($3*$2)/1024;
	my ($avail_bytes) = $avail;
	my ($capper) = int(($3/$1)*100);
	my ($mountpt) = "\\\\$remote\\$share";

	#Check $warn and $crit for type (%/M/G) and set up for tests
	#P = Percent, K = KBytes
	my $warn_type;
	my $crit_type;
	if ($warn =~ /^([0-9]+$)/) {
		$warn_type = "P";
	} elsif ($warn =~ /^([0-9]+)k$/) {
		my ($warn_type) = "K";
		$warn = $1;
	} elsif ($warn =~ /^([0-9]+)M$/) {
		$warn_type = "K";
		$warn = $1 * 1024;
	} elsif ($warn =~ /^([0-9]+)G$/) {
		$warn_type = "K";
		$warn = $1 * 1048576;
	}
	if ($crit =~ /^([0-9]+$)/) {
		$crit_type = "P";
	} elsif ($crit =~ /^([0-9]+)k$/) {
		$crit_type = "K";
		$crit = $1;
	} elsif ($crit =~ /^([0-9]+)M$/) {
		$crit_type = "K";
		$crit = $1 * 1024;
	} elsif ($crit =~ /^([0-9]+)G$/) {
		$crit_type = "K";
		$crit = $1 * 1048576;
	}

	if (int($avail / 1024) > 0) {
		$avail = int($avail / 1024);
		if (int($avail /1024) > 0) {
			$avail = (int(($avail / 1024)*100))/100;
			$avail = $avail."G";
		} else {
			$avail = $avail."M";
		}
	} else {
		$avail = $avail."K";
	}

#print ":$warn:$warn_type:\n";
#print ":$crit:$crit_type:\n";
#print ":$avail:$avail_bytes:$capper:$mountpt:\n";
	if ((($warn_type eq "P") && (100 - $capper) < $warn) || (($warn_type eq "K") && ($avail_bytes > $warn))) { 
		$answer = "Disk ok - $avail ($capper%) free on $mountpt\n";
	} elsif ((($crit_type eq "P") && (100 - $capper) < $crit) || (($crit_type eq "K") && ($avail_bytes > $crit))) {
		$state = "WARNING";
		$answer = "Only $avail ($capper%) free on $mountpt\n";
	} else {
		$state = "CRITICAL";
		$answer = "Only $avail ($capper%) free on $mountpt\n";
	}
} else {
	$answer = "Result from smbclient not suitable\n";
	$state = "UNKNOWN";
	foreach (@lines) {
		if (/Access denied/) {
			$answer = "Access Denied\n";
			$state = "CRITICAL";
			last;
		}
		if (/(Unknown host \w*)/) {
			$answer = "$1\n";
			$state = "CRITICAL";
			last;
		}
		if (/(You specified an invalid share name)/) {
			$answer = "Invalid share name \\\\$remote\\$share\n";
			$state = "CRITICAL";
			last;
		}
	}
}


print $answer;
#print "$state\n";
exit $ERRORS{$state};

sub usage {
	print "Minimum arguments not supplied!\n";
	print "\n";
	print "Perl Check SMB Disk plugin for NetSaint\n";
	print "Usage: check_disk_smb.pl <host> <share> <user> <passwd> [<warn> [<crit>]]\n";
	print "\n";
	print "<host> = Hostname for the server.  This needs to the the NetBIOS name not the IP.\n";
	print "<share> = Share name to be tested.\n";
	print "<user> = Username to be used to log in to server.\n	Defaults to \"guest\".\n";
	print "<passwd> = Password to be used to log in to server.\n	Defaults to \"guest\".\n";
	print "<warn> = Percent of used space at which a warning message will be generated.\n	Defaults to 85.\n";
	print "<crit> = Percent of used space at which a critical message will be generated.\n	Defaults to 95.\n";
	exit $ERRORS{"UNKNOWN"};
}

