#!/usr/bin/perl -wT

# (c)1999 Ian Cass, Knowledge Matters Ltd.
# Read the GNU copyright stuff for all the legalese
#
# Check NTP time servers plugin. This plugin requires the ntpdate utility to
# be installed on the system, however since it's part of the ntp suite, you 
# should already have it installed.
#
# Nothing clever done in this program - its a very simple bare basics hack to
# get the job done.
#
# Things to do...
# check @words[9] for time differences greater than +/- x secs & return a
# warning.
#
# (c) 1999 Mark Jewiss, Knowledge Matters Limited
# 22-9-1999, 12:45
#
# Modified script to accept 2 parameters or set defaults.
# Now issues warning or critical alert is time difference is greater than the 
# time passed.
#
# These changes have not been tested completely due to the unavailability of a
# server with the incorrect time.
#
# (c) 1999 Bo Kersey, VirCIO - Managed Server Solutions <bo@vircio.com>
# 22-10-99, 12:17
#
# Modified the script to give useage if no parameters are input.
#
# Modified the script to check for negative as well as positive 
# time differences.
#
# Modified the script to work with ntpdate 3-5.93e Wed Apr 14 20:23:03 EDT 1999
#
# Modified the script to work with ntpdate's that return adjust or offset...
#


require 5.004;
use POSIX;
use strict;

$ENV{PATH} = "";
$ENV{BASH_ENV} = "";
$ENV{ENV} = "";

sub usage;

my $TIMEOUT = 10;

my %ERRORS = ('UNKNOWN' , '-1',
	      'OK' , '0',
	      'WARNING', '1',
	      'CRITICAL', '2');

my $remote = shift || &usage(%ERRORS);
my $warning = shift || 60;
my $critical = shift || 120;

my $state = "OK";
my $answer = undef;
my $offset = undef;
my $ntpret = 0; 

# Just in case of problems, let's not hang NetSaint
$SIG{'ALRM'} = sub {
     print ("ERROR: No response from ntp server (alarm)\n");
     exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);

if ($warning =~ m/([0-9]+)/) {
    $warning = $1;
} else {
     print ("NTP UNKNOWN: warning threshold must be a non-negative integer\n");
     exit $ERRORS{"UNKNOWN"};
}

if ($critical =~ m/([0-9]+)/) {
    $critical = $1;
} else {
     print ("NTP UNKNOWN: critical threshold must be a non-negative integer\n");
     exit $ERRORS{"UNKNOWN"};
}

if ($remote =~ m/(\d+\.\d+\.\d+\.\d+)/){
    $remote = $1;
} elsif ($remote =~ m/([a-zA-Z][-a-zA-Z0-9]*(\.[a-zA-Z][-a-zA-Z0-9]*)*)/) {
    $remote = $1;
} else {
     print ("NTP UNKNOWN: Incorrect hostname syntax\n");
     exit $ERRORS{"UNKNOWN"};
}

open (NTPDATE, "/usr/sbin/ntpdate -q $remote |"); 

while (<NTPDATE>) {
    ($_ =~ m/(offset|adjust)\s+(\-?\d+\.\d+)/i) && ($offset=$2);
}

close(NTPDATE);

$ntpret = ($? >> 8);

if (defined $offset) {
    if (abs($offset) > $warning) {
	if (abs($offset) > $critical) {
	    $state = "CRITICAL";
	    $answer = ": Time difference $offset seconds greater than +/- $critical sec\n";
	} else {
	    $state = "WARNING";
	    $answer = ": Time difference $offset seconds greater than +/- $warning sec\n";
	} 
    } else {
	$state = "OK";
	$answer = ": Time Difference $offset seconds\n";
    }
} else {
    $state = "UNKNOWN";
    $answer = ": Invalid format returned from ntpdate\n";
} 

unless ($ntpret == 0) { 
    $state="CRITICAL"; 
    $answer=": No server running\n"; 
}

print ("$state$answer");
exit $ERRORS{$state};

sub usage {
	print "Minimum arguments not supplied!\n";
	print "\n";
	print "Perl Check NTP plugin for NetSaint\n";
	print "Copyright (c) 1999 Bo Kersey\n";
	print "\n";
	print "Usage: check_ntp.pl <host> [<warn> [<crit>]\n";
	print "\n";
	print "<warn> = Clock offset in seconds at which a warning message will be generated.\n	Defaults to 60.\n";
	print "<crit> = Clock offset in seconds at which a critical message will be generated.\n	Defaults to 120.\n\n";
	exit $ERRORS{"UNKNOWN"};
}









